using System;
using System.IO;
using Foxit.PDF;
using Foxit.PDF.PageElements;

namespace Web_CSharp.GeneratorExamples
{
	public partial class SourceView : System.Web.UI.Page
	{
		#region Member Variables

		//////////////////////////////////////////////////
		// Begin Setting Region
		// These settings are for editable. Modify the settings below based on your requirements

		private static Font font = Font.Courier;
		private const float fontSize = 9.0f;
		private static PageDimensions pageDimensions = new PageDimensions( PageSize.Letter, PageOrientation.Landscape, 36.0f );
		private static float left = font.GetTextWidth( "0000", fontSize ) + 6;
		private static float top = Font.Helvetica.GetDefaultLeading( 10.0f ) + 3.0f;
		private static float bottom = pageDimensions.Body.Height - 12;
		private static float lineWidth = pageDimensions.Body.Width - left;
		private const string tabSpaces = "    ";
		private static Template template;
        private const string footerText = "<p align=\"center\">This PDF was generated in real-time using <i>Foxit</i><b>PDF</b>; Generator for .NET. http://www.foxitsoftware.com/products/sdk/</p>";
		
		//////////////////////////////////////////////////
		
		private int lineNum = 1;
		private string currentUrl = "";
		private string aspxUrl = "";
		private string codeBehindUrl = "";
		private float currentPosition = 0;
		private Foxit.PDF.Page page = null; 
		private Document document = null; 
		
		#endregion

		#region Constructors

		static SourceView()
		{
			template = new Template();
			template.Elements.Add( new Line( 0, top - 1, pageDimensions.Body.Width, top - 1 ) );
			template.Elements.Add( new Line( 0, top - 1, pageDimensions.Body.Width, top - 1 ) );
			template.Elements.Add( new Line( left - 3, top - 1, left - 3, bottom ) );
			template.Elements.Add( new Line( 0, bottom, pageDimensions.Body.Width, bottom ) );
			template.Elements.Add( new FormattedTextArea( footerText, 0, bottom + 1.0f, pageDimensions.Body.Width, 10, FontFamily.Helvetica, 10, false ) );
            template.Elements.Add(new Link(0, bottom + 1, pageDimensions.Body.Width, 14, new UrlAction("http://www.foxitsoftware.com/products/sdk/?lf=SrcView")));
		}
		#endregion

		#region Page Load

		protected void Page_Load(object sender, System.EventArgs e)
		{
			// Retrieve the class query string variable
			string classPath = Request.QueryString["class"];
			if( classPath == null ) 
			{
				Response.Write( " No File Selected ");
				return;
			}

			if( classPath.IndexOf( "/" ) >= 0 || classPath.IndexOf( "\\" ) >= 0) 
			{
				Response.Write("For security reasons slashes are not allowed in the class name ");
				return;
			}

			// Set the file paths based on the class varialbe
			aspxUrl = Server.MapPath( classPath + ".aspx" );
			codeBehindUrl =  aspxUrl + ".cs" ;
			if( !File.Exists( aspxUrl ) || !File.Exists( codeBehindUrl ) )
			{
				Response.Write("Class name does not exist");
				return;
			}

			// Create a document and set it's properties
			document = new Document();
			document.Creator = "SourceView.aspx";
			document.Author = "Foxit Software";
			document.Title = "Source View";

			// Set the template to the document
			document.Template = template;
			
			// Add a bookmark to the ASPX page
			document.Outlines.Add( "ASPX Page", new XYDestination( 1, 0, 0 ) );
			
			// Add the source of the ASPX page to the document
			AddSource( aspxUrl );
			
			// Add a bookmark to the code behind page
			document.Outlines.Add( "Code Behind Page", new XYDestination( document.Pages.Count + 1, 0, 0 ) );
			
			// Add the source of the code behind page to the document
			AddSource( codeBehindUrl );
			
			// Outputs the document to the current web page
			document.DrawToWeb( "SourceView.pdf" );
		}
		#endregion

		#region Private Methods

		private void AddSource( string filePath )
		{
			// reset the line number
			lineNum = 1;
			currentUrl = System.IO.Path.GetFileName(filePath);

			// Add the initial source page to the document
			AddSourcePage();
			
			// Open file
			StreamReader sourceCode = new StreamReader( filePath );
			
			// Loop over each line
			string lineText;
			while( ( lineText = sourceCode.ReadLine() ) != null )
			{
				AddSourceLine( lineText );
			}
			
			// Close the file
			sourceCode.Close();
		}

		private void AddSourceLine( string lineText )
		{
			// Clean up text
			string trimmedLineText = lineText.Trim();
			TextArea line = new TextArea( lineText.Replace( "\t", tabSpaces ), left, currentPosition, lineWidth, bottom - currentPosition, font, fontSize );
			float fltHeight = line.GetRequiredHeight();

			// Set line color if it is a comment
			if( trimmedLineText.StartsWith( "//" ) || trimmedLineText.StartsWith( "'" ) ) line.TextColor = RgbColor.Green;
			
			// Add a new page if line does not fit
			if( currentPosition + fltHeight + 2 > bottom )
			{
				AddSourcePage();
				line.Y = currentPosition;
			}
			
			// Add line number and line text to page
			page.Elements.Add( new Label( lineNum.ToString(), 0, currentPosition, left - 6, fontSize, font, fontSize, TextAlign.Right, RgbColor.Gray ) );
			page.Elements.Add( line );
			
			// increment for next line
			currentPosition += fltHeight;
			lineNum++;
		}

		private void AddSourcePage()
		{
			// Adds a new page to the document
			page = new Foxit.PDF.Page( pageDimensions );
			
			// Adds the file name to the page
			TextArea url = new TextArea( currentUrl, 0, 0, page.Dimensions.Body.Width, 100, Font.Helvetica, 10.0f, RgbColor.DarkBlue );
			page.Elements.Add( url );
			
			// Resets the current position for the page
			currentPosition = top + 1.0f;
			
			// Adds the page to the document
			document.Pages.Add( page );
		}

		#endregion

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
		}
		#endregion
	}
}
