using System;
using Foxit.PDF;
using Foxit.PDF.PageElements;
using Foxit.PDF.PageElements.BarCoding;

namespace Web_CSharp.GeneratorExamples
{
	public partial class USEnvelope : System.Web.UI.Page
	{

		protected void Page_Load(object sender, System.EventArgs e)
		{
			if( IsPostBack )
			{
				// Create a document and set it's properties
				Foxit.PDF.Document document = new Foxit.PDF.Document();
				document.Creator = "USEnvelope.aspx";
				document.Author = "Foxit Software";
				document.Title = "US Envelope";

				// Create a page and add it to the document.
				Foxit.PDF.Page page = new Foxit.PDF.Page( PageSize.Envelope10, PageOrientation.Landscape, 18 );
				document.Pages.Add( page );
				
				// Uncomment the line below to show a layout grid.
				//page.Elements.Add( new LayoutGrid() );
				
				// Add page elements to the page
				page.Elements.Add( new Image( MapPath( "../Images/FoxitLogo.png" ), 0, 2, 0.16f ) );
				page.Elements.Add( new TextArea( GetFromAddress(), 50, 0, 350, 80, Font.Helvetica, 10 ) );
				page.Elements.Add( new TextArea( GetToAddress(), 300, 140, 360, 100, Font.Helvetica, 12 ) );
				page.Elements.Add( new Postnet( txtToZip.Text, 300, 248 ) );
				
				// Check for an error
				try
				{
					document.DrawToWeb( "USEnvelope.pdf" );
				}
				catch( InvalidValueBarCodeException )
				{
					// Show the Postnet value error
					ShowErrorDocument( "Invalid To Zipcode. To zipcode must be a valid US zipcode.", "" );
				}
				catch( Exception excGeneral )
				{
					// Show a general error
					ShowErrorDocument( excGeneral.Message, excGeneral.StackTrace );
				}
				finally
				{
					Visible = false;
				}
			}
		}

		private void ShowErrorDocument( string message, string stackTrace )
		{
			// Shows the error in a PDF document
			Response.Clear();
			Document document = new Document();
			Foxit.PDF.Page page = new Foxit.PDF.Page( PageSize.Letter, PageOrientation.Portrait );

			TextArea messageArea = new TextArea( message, 0, 0, 512, 36, Font.Helvetica, 18, RgbColor.Red );
			messageArea.Height = messageArea.GetRequiredHeight();
			page.Elements.Add( messageArea );
			
			if( stackTrace.Trim().Length > 0 )
			{
				float stackTraceTop = messageArea.Y + messageArea.Height + 20;
				TextArea stackTraceTA = new TextArea( stackTrace, 0, stackTraceTop, 512, 692 - stackTraceTop, Font.Helvetica, 10 );
				page.Elements.Add( new Foxit.PDF.PageElements.Label( "StackTrace:", 0, stackTraceTop - 12, 512, 12, Font.HelveticaBold, 10 ) );
				page.Elements.Add( stackTraceTA );
			}
			document.Pages.Add( page );
			document.DrawToWeb();
		}

		private string GetFromAddress()
		{
			// Returns a formatted From Address
			return GetAddress( txtFromName.Text, txtFromAddress.Text, txtFromAddress2.Text, txtFromCity.Text, txtFromState.Text, txtFromZip.Text );
		}

		private string GetToAddress()
		{
			// Returns a formatted To Address
			return GetAddress( txtToName.Text, txtToAddress.Text, txtToAddress2.Text, txtToCity.Text, txtToState.Text, txtToZip.Text );
		}

		private string GetAddress( string name, string address, string address2, string city, string state, string zip )
		{
			// Formats a US address
			string myAddress = name.Trim() + "\n";
			myAddress += address.Trim() + "\n";
			if( address2.Trim().Length > 0 ) myAddress += address2.Trim() + "\n";
			myAddress += city.Trim() + ", " + state.Trim() + "  " + zip.Trim();
			return myAddress;
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
		}
		#endregion
	}
}
